<?php
/*
 PHP script by Ken True, webmaster@saratoga-weather.org
 rss-advisory.php  version 1.00 - 17-Apr-2006
 http://saratoga-weather.org/scripts.php

 You may copy/modify/use this script as you see fit,
 no warranty is expressed or implied.

 Version 1.01 - 16-Aug-2006 - added cacheing for RSS feed
 Version 1.02 - 08-Dec-2007 - added time formatting for lastBuildDate
 Version 1.03 - 17-Dec-2007 - added safety features from Mike Challis - http://www.642weather.com/weather/
                              fixed XHTML 1.0-Strict output, improved debug code,
                              added file-mode include functions
 Version 1.04 - 22-Jan-2008 - added graceful failure code from MChallis - http://www.642weather.com/weather/
 Version 1.05 - 27-Jan-2008 - added many features from Mike Challis, rename to rss-advisory.php
                              accommodates a different CSS style for when there is no advisory,
                              backwards compatible with old include methods of you are not going to use the
                              CSS style switch for no advisory / advisory
                              NOTE: to use the new feature (CSS style switch for no advisory / advisory)
                              use the new php include params and add the sample css (see settings notes),
                              added invalid zone error code
                              added option to lowercase the advisories
 Version 1.06 - 02-Feb-2008 - added flyer usage feature from from Mike Challis.
                              when used for a printable flyer, advisory titles are shown without URL links
                              added two new settings for flyer (off by default)
                              Live example: http://www.642weather.com/weather/local-flier.php
                              (the live example only shows advisories when they are currently active)
 Version 1.07 - 04-Feb-2008 - added support for Carterlake/AJAX/PHP template set
 Version 1.08 - 17-Mar-2008 - added improved cache file error handling
 Version 1.09 - 25-Mar-2008 - Corrected the HTML comments on some error message text
 Version 1.10 - 03-Jul-2009 - added PHP5 timezone setting support + PHP_SELF checking
*/
$Version = 'V1.10 03-Jul-2009';
/*

This script gets the current RSS Zone Watch/Warning/Advisory/Statement
from www.weather.gov/alerts/ for the selected Zone and provides either
a summary (titles only, with links) or details.  It returns
'There are no active watches, warnings or advisories for zone CAZ513'
if there are no cited hazards in the Zone CAZ513.

 output: creates XHTML 1.0-Strict HTML page (default)

 Options on URL:
      inc=Y           -- returns only the body code for inclusion
                         in other webpages.  Omit to return full HTML.
      summary=Y       -- returns only the titles of the cited hazards
                         N.N or greater.
      zone=ssZnnn     -- select the Zone to use.  Pick the Zone from the
                          RSS feed list at http://www.weather.gov/alerts/
                         and use the RSS County/Zone for your area.
     detailpage=xxx   -- set your page of full advisories ie: detailpage=advisories.php
     cache=refresh    -- reload the cache

 Example URL:
  http://your.website/rss-advisory.php?zone=CAZ513
  would return data for hazards in Zone=CAZ513
  which is Santa Clara County, California

#########
# PHP Include Usage:
#########
  You can include this with summary output only
  or you can include it with full advisory text output:

There are two ways of including the script in your pages depending on whether your hoster has
allow_url_include option set to on or off. The URL method only works if allow_url_fopen
is on in your PHP settings. The File method usually always works.

#########
# Include with Summary output usage:
# Live example: http://www.642weather.com/weather/index.php
#########
to include this with Summary output only, put the following code in your php page inside php tags:
# (URL method): (do not use both URL and File methods at once!)

include 'http://your.website/rss-advisory.php?zone=WAZ021&inc=Y&summary=Y&noprint=1&detailpage=advisories.php';
 if (preg_match("|There are no active|i",$advisory_html) ||
     preg_match("|Advisory Information Unavailable|i",$advisory_html)) {
       echo '<div class="advisoryBoxnoactive">' .$advisory_html .'</div>';
 }else{
       echo '<div class="advisoryBox">' . $advisory_html .'</div>';
 }

# (File method): (do not use both URL and File methods at once!)

 $_REQUEST['zone'] = 'WAZ021';
 $_REQUEST['inc'] = 'Y';
 $_REQUEST['summary'] = 'Y';
 $_REQUEST['noprint'] = 1;
 include 'rss-advisory.php';
 if (preg_match("|There are no active|i",$advisory_html) ||
     preg_match("|Advisory Information Unavailable|i",$advisory_html)) {
       echo '<div class="advisoryBoxnoactive">' .$advisory_html .'</div>';
 }else{
       echo '<div class="advisoryBox">' . $advisory_html .'</div>';
 }

#########
# Include with Full advisory output usage:
# Live example: http://www.642weather.com/weather/advisories.php
#########
to include this with Full advisory output, put the following code in your php page inside php tags:
# (URL method): (do not use both URL and File methods at once!)

include 'http://your.website/rss-advisory.php?zone=WAZ021&inc=Y&noprint=1&detailpage=advisories.php';

 if (preg_match("|There are no active|i",$advisory_html) ||
     preg_match("|Advisory Information Unavailable|i",$advisory_html)) {
       echo '<div class="advisoryBoxnoactive">' .$advisory_html .'</div>';
 }else{
       echo $advisory_html;
 }
# (File method): (do not use both URL and File methods at once!)

 $_REQUEST['zone'] = 'WAZ021';
 $_REQUEST['inc'] = 'Y';
 $_REQUEST['noprint'] = 1;
 include 'rss-advisory.php';
 if (preg_match("|There are no active|i",$advisory_html) ||
     preg_match("|Advisory Information Unavailable|i",$advisory_html)) {
       echo '<div class="advisoryBoxnoactive">' .$advisory_html .'</div>';
 }else{
       echo $advisory_html;
 }

#########
# CSS sample:
#########
put this css style code in your css file (modify the colors if you like)
see live sample at http://www.642weather.com/weather/index.php
forum discussion page:
http://www.weather-watch.com/smf/index.php/topic,29274.0.html

.advisoryBox {
  color: black;
  font-size: 12px;
  text-align: center;
  background-color: #FFE991;
  margin: 0 0 0 0;
  padding: .5em 0em .5em 0em;
  border: 1px dashed rgb(34,70,79);
}
.advisoryBoxnoactive {
  color: black;
  font-size: 12px;
  text-align: center;
  background-color: white;
  margin: 0 0 0 0;
  padding: .5em 0em .5em 0em;
  border: 1px dashed rgb(34,70,79);
}

##############
# begin settings -------------------------------------------------
##############
*/
//  change $myDefaultZone default(below) to your stations county zone
//  other settings are outlined below, and are optional

$myDefaultZone = 'WAZ021'; // <== change this to your zone

$hurlURL = 'advisories.php'; // <== change this default to your webpage
                                 // to open for details

$HD = 'h2';  // <== type of heading for advisories <$HD>...</$HD>

$doLowerCase = false;  // <== change to true to lowercase the advisories (new in 1.05)

    // cacheName is name of file used to store cached current conditions
$cacheName = 'rss-advisory.txt';  // used to store the file so we
    //                                        don't have to fetch it each time
    // note: the real cache name will automatically be the above name with the Zone included
    //   ie:  rss-advisory.txt => rss-advisory-CAZ513.txt

$ourTZ = 'PST8PDT';  //NOTE: this *MUST* be set correctly to
   // translate UTC times to your LOCAL time for the displays.
   //  http://saratoga-weather.org/timezone.txt  has the list of timezone names
   //  pick the one that is closest to your location and put in $ourTZ like:
   //    $ourTZ = 'America/Los_Angeles';  // or
   //    $ourTZ = 'Europe/Brussels';
   // also available is the list of country codes (helpful to pick your zone
   // from the timezone.txt table
   //  http://saratoga-weather.org/country-codes.txt : list of country codes

//$timeFormat = 'D, Y-m-d H:i:s T';  // Fri, 2006-03-31 14:03:22 TZone
$timeFormat = 'D, d-M-Y h:ia T';  //  Sun, 27-Jan-2008 06:14am TZone

$refetchSeconds = 600; // refetch every nnnn seconds (600=10 minutes)

$flyer = false;  // <== change to true when using for a printable flyer page (new in 1.06)
                // when used for a printable flyer, advisory titles are shown without URL links

   //flyerpromo will show on your printable flyer page just after an advisory headline
   // example: High Wind Warning - South Washington Coast (Washington)(view at www.642weather.com)
$flyerpromo = '<span style="font-size: 11px;"> (view at www.642weather.com)</span>';

##############
# end of settings -------------------------------------------------
##############
// overrides from Settings.php if available
global $SITE;
if (isset($SITE['noaazone'])) 	{$myDefaultZone = $SITE['noaazone'];}
if (isset($SITE['hurlURL'])) 	{$hurlURL = $SITE['hurlURL'];}
if (isset($SITE['timeFormat'])) {$timeFormat = $SITE['timeFormat'];}
if (isset($SITE['tz'])) 		{$ourTZ = $SITE['tz'];}
// end of overrides from Settings.php if available

#####                                                                      #####
# Do not alter any code below this point in the script or it may not run properly.
#####                                                                      #####
// error_reporting(E_ALL);
if (!isset($PHP_SELF)) {$PHP_SELF = $_SERVER['SCRIPT_NAME']; } // needed for some PHP installations

if(!isset($DefaultZone)) {
$DefaultZone = $myDefaultZone;
}
if(!isset($hurlURL)) {
$hurlURL = $PHP_SELF;
}

// Establish timezone offset for time display
# Set timezone in PHP5/PHP4 manner
  if (!function_exists('date_default_timezone_set')) {
	putenv("TZ=" . $ourTZ);
#	$Status .= "<!-- using putenv(\"TZ=$ourTZ\") -->\n";
    } else {
	date_default_timezone_set("$ourTZ");
#	$Status .= "<!-- using date_default_timezone_set(\"$ourTZ\") -->\n";
   }

//  get request parameters
if ( ! isset($_REQUEST['zone']) ) {
        $_REQUEST['zone']="$DefaultZone";
        }
$Zone = $_REQUEST['zone'];

// full html output or just contents (inc=Y)?
if ( ! isset($_REQUEST['inc']) ) {
        $_REQUEST['inc']='';
}
$includeOnly = $_REQUEST['inc']; // any nonblank is ok
if ($includeOnly) {$includeOnly = 'Y';}
if ( isset($_REQUEST['summary'])) {
  $doSummary = TRUE;
  } else {
  $doSummary = FALSE;
}
if (isset($_REQUEST['detailpage']) ) {
// new, safer way of setting detailpage
  $t = parse_url(urldecode($_REQUEST['detailpage']));
  $Status .= "<!-- detailpage\n" . print_r($t,true) . " -->\n";
  if (isset($t['path'])) {$hurlURL = $t['path'];}
}
if (isset($_REQUEST['sce']) and $_REQUEST['sce'] == 'view' and !$includeOnly) {
   //--self downloader --
   $filenameReal = __FILE__;
   $download_size = filesize($filenameReal);
   header('Pragma: public');
   header('Cache-Control: private');
   header('Cache-Control: no-cache, must-revalidate');
   header("Content-type: text/plain");
   header("Accept-Ranges: bytes");
   header("Content-Length: $download_size");
   header('Connection: close');
   readfile($filenameReal);
   exit;
 }
if (isset($_REQUEST['cache']) and strtolower($_REQUEST['cache']) == 'refresh') {
  $refetchSeconds = 1;  // set short period for refresh of cache
}
// fix up zone if need be.... only alpha-numeric and uppercase needed
$Zone = strtoupper(preg_replace('/[^A-Za-z0-9]/i','',$Zone));
$RSS_URL = "http://www.weather.gov/alerts/wwarssget.php?zone=$Zone";
// begin code -----------------------------------------------------------
global $Status;
# mchallis bof added for flyer
global $advcount;
$advcount = 0;
# mchallis eof added for flyer

$t = pathinfo($PHP_SELF);
$Program = $t['basename'];

$cacheName = preg_replace('|\.txt$|i',"-$Zone.txt",$cacheName);

$Status = "<!-- $Program - $Version -->\n";

if (file_exists($cacheName) and filemtime($cacheName) + $refetchSeconds > time()) {  //600=10 min
	$age = time() - filectime($cacheName);
	$nextFetch = $refetchSeconds - $age;
    $Status .= "<!-- using cached version from $cacheName - age=$age secs. Next fetch in $nextFetch secs. -->\n";
	$fp = fopen($cacheName,"r") or die("Unable to open cache $cacheName for RSS data.");
	$html = fread($fp,filesize($cacheName));
	fclose($fp);
	if (strlen($html) < 20) {
	  $Status .= "<!-- Warning: cache file size=" . strlen($html) . " is too small.. XML data not found. -->\n";
	}
} else {
    $Status .= "<!-- getting cache from $RSS_URL -->\n";
	$xml = GrabURLWithoutHanging($RSS_URL);
	list($junk,$html) = explode("\r\n\r\n",$xml,2);
	if (strlen($html) > 20) {
	  $fp = fopen($cacheName, "w");
      if ($fp) {
        $write = fputs($fp, $html);
        fclose($fp);
		$Status .= "<!-- cache saved to $cacheName -->\n";
      } else {
	    $Status .= "<!-- unable to write $cacheName -->\n";
      }
	} else {
	   $Status .= "<!-- HTML fetch length = " . strlen($html) . " ; Cache not saved. -->\n";
	   $Status .= "<!-- headers " . $junk . " -->\n";
	}
}

$insideitem = false;
$tag = "";
$title = "";
$description = "";
$link = "";
$lastBuildDate = "";
$tracking = "";
$Summary = "";
$WLink = 0;

# mchallis modified for all output to be collect into a string instead of using print statements.
//  Main Code Start
// all output from program comes from here
$advisory_html = advisory_main_code();

if (!$includeOnly || !isset($_REQUEST['noprint'])) {
   echo $advisory_html;
   $advisory_html = "<!-- Warning: Trying to include \$advisory_html without the noprint=1 parameter -->\n";
}

// ----------------------------functions -----------------------------------

# mchallis added function
function advisory_main_code() {
  global $insideitem, $tag, $title, $description, $link, $lastBuildDate, $tracking, $Summary;
  global $doSummary, $Zone, $WLink, $cacheName, $includeOnly, $string, $Status, $doLowerCase, $flyer;

   if (! $includeOnly) {  // omit HTML headers if doing inc=Y
     $string .= '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
<title>RSS Advisory Script</title>
</head>

<body style="background-color:#FFFFFF; font-family:Verdana, Arial, Helvetica, sans-serif; font-size:12px;">
';
if(!$includeOnly and $doSummary) { $string .=  "<p>\n"; } // for XHTML 1.0-Strict validation
} // end .. only printed if full html needed

  $string .= $Status;

  $xml_parser = xml_parser_create();
  xml_set_element_handler($xml_parser, "startElementAdvisory", "endElementAdvisory");
  xml_set_character_data_handler($xml_parser, "characterDataAdvisory");

   # mchallis added graceful error function instead of using "or die"
   # when the script "dies" your web page will be stuck right where the error was. no good
   if (!$fp = fopen($cacheName,"r")) {
      $string .= gracefulerror('Error reading RSS data cache file.');
      return $string;
   }

   while ($data = fread($fp, 4096))
         if(!xml_parse($xml_parser, $data, feof($fp))){
              # mchallis added graceful error function instead of using "or die"
              # when the script "dies" your web page will be stuck right where the error was. no good
          $string .= gracefulerror("$PHP_SELF XML error: " . xml_error_string(xml_get_error_code($xml_parser)). ' at line '.
                 xml_get_current_line_number($xml_parser).': '. htmlspecialchars(strip_tags(trim($data))) );

          fclose($fp);
          xml_parser_free($xml_parser);
          return $string;
         }
         fclose($fp);
         xml_parser_free($xml_parser);

   if ($doSummary) {
     $string .= $Summary;
    # mchallis modified for flyer usage
     if ($WLink && !$flyer) {
          $string .= "Click on link";
          if ($WLink > 1) {$string .= "s";}
          $string .= " above to see details on the $WLink NOAA advisor";
          if ($WLink > 1) {$string .= "ies"; } else { $string .= "y"; };
          $string .= " for zone $Zone.\n";
     }
   }
if (! $includeOnly) {
$string .= "\n<!-- Zone='$Zone' -->\n";
// $string .= $tracking;
if(!$includeOnly and $doSummary) { $string .=  "</p>\n"; } // for XHTML 1.0-Strict validation
$string .= '</body>
</html>
';
   } // end - only printed if full html wanted (no inc=Y)
return $string;
} // # end mchallis function - advisory_main_code


// get contents from one URL and return as string
 function GrabURLWithoutHanging($url) {
// thanks to Tom at Carterlake.org for this script fragment
  global $Status;
  $UA = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.8.0.6) Gecko/20060728 Firefox/1.5.0.6';
   // Set maximum number of seconds (can have floating-point) to wait for feed before displaying page without feed
   $numberOfSeconds=4;

   // Suppress error reporting so Web site visitors are unaware if the feed fails
   error_reporting(0);

   // Extract resource path and domain from URL ready for fsockopen
   $FullUrl = $url;
   $url = str_replace("http://","",$url);
   $urlComponents = explode("/",$url);
   $domain = $urlComponents[0];
   $resourcePath = str_replace($domain,"",$url);
   $Status .= "<!-- GET $resourcePath HTTP/1.0 \n      Host: $domain -->\n";
   $time_start = microtime_float();

   // Establish a connection
   $socketConnection = fsockopen($domain, 80, $errno, $errstr, $numberOfSeconds);

   if (!$socketConnection)
       {
       // You may wish to remove the following debugging line on a live Web site
       $Status .= "<!-- Network error: $errstr ($errno) -->\n";
       }    // end if
   else    {
       $xml = '';
       fputs($socketConnection, "GET $resourcePath HTTP/1.0\r\nHost: $domain\r\nUser-agent: $UA\r\nConnection: close\r\n\r\n");

       // Loop until end of file
       while (!feof($socketConnection))
           {
           $xml .= fgets($socketConnection, 4096);
           }    // end while

       fclose ($socketConnection);

       }    // end else
   $time_stop = microtime_float();
   $total_time += ($time_stop - $time_start);
   $time_fetch = sprintf("%01.3f",round($time_stop - $time_start,3));
   $Status .= "<!-- Time to fetch: $time_fetch sec -->\n";

//   list($headers,$html) = explode("\r\n\r\n",$xml,2);
   return($xml);

}    // end GrabURLWithoutHanging


// function for XML parsing .. invoked at start of XML element
function startElementAdvisory($parser, $name, $attrs) {
        global $insideitem, $tag, $title, $description, $link, $lastBuildDate, $Summary, $doSummary;
        if ($insideitem) {
                $tag = $name;
        } elseif ($name == "ITEM") {
                $insideitem = true;
        }
        if ($name == "LASTBUILDDATE") {
           $tag = $name;
        }
}

// function for XML parsing .. invoked at end of XML element
// bulk of the work is done here
function endElementAdvisory($parser, $name) {
  global $insideitem, $tag, $title, $description, $link, $lastBuildDate, $tracking, $Summary, $advcount;
  global $doSummary, $Zone, $hurlURL, $WLink, $HD, $string, $timeFormat,$doLowerCase,$flyer,$flyerpromo;
        if ($name == "ITEM") {
            if (preg_match("|There are no active|i",$description)) {
                  if (! $doSummary) {
                    $string .= '<p>'.htmlspecialchars(trim($description)). " for zone $Zone.</p>";
                  }
                   # mchallis modified
                   # "There are no active watches, warnings or advisories for $Zone"
                  $Summary = htmlspecialchars(trim($description)) . " for zone $Zone.";
            } else {
                   $WLink++;
                   $description = preg_replace("|<br>|s","<br />\n",$description);
                   $description = preg_replace("|href=http([^>]+)>|is","href=\"http\\1\">",$description);
                   # mchallis added security feature
                   $title = strip_tags($title, '<br>');
                   # mchallis added security feature
                   $description = strip_tags($description, '<b><br><a>');
                   if (! $doSummary) {
                         $string .= "<$HD><a name=\"WL$WLink\"></a><a href=\"".trim($link)."\">".htmlspecialchars(trim($title))."</a></$HD>\n";
                         $buildDate = strtotime(substr($lastBuildDate,5) . ' GMT');
                         $buildDateText = date($timeFormat,$buildDate);
                         $string .= "<!-- '$lastBuildDate GMT'='$buildDateText' -->\n";
                         $string .= "<p>Updated: <b>".$buildDateText."</b><br />\n";
                                                 if($doLowerCase) {
                                                   $string .= trim(strtolower($description));
                                                 } else {
                                                   $string .= trim($description);
                                                 }
                                                 $string .= "</p>\n";
                   }
                   # mchallis modified for flyer usage
                   if ($flyer) {
                   $Summary .= '<span style="color: red;"><b>' . htmlspecialchars(trim($title)) . "</b></span>$flyerpromo<br />\n";
                   }else{
                         $Summary .= "<span style=\"color: red;\"><a href=\"$hurlURL?zone=$Zone#WL$WLink\"><b>" .
                         htmlspecialchars(trim($title)) . "</b></a></span><br />\n";
                   }
                   $advcount++;
            }
                $title = "";
                $description = "";
                $link = "";
                $insideitem = false;
        } // end insideitem
        if ($name == "LASTBUILDDATE") {
           $tag = '';
        }
        $tracking .= "<!-- tag: '$name' -->\n";
}

// extract character data from within an XML tag
function characterDataAdvisory($parser, $data) {
        global $insideitem, $tag, $title, $description, $link, $lastBuildDate, $Summary, $doSummary;
        if ($tag == "LASTBUILDDATE") {
                $lastBuildDate .= $data;
        }
        if ($insideitem) {
           switch ($tag) {
                case "TITLE":
                $title .= $data;
                break;
                case "DESCRIPTION":
                $description .= $data;
                break;
                case "LINK":
                $link .= $data;
                break;
           } // end switch
        } // end insideitem
}

// ------------------------------------------------------------------
function microtime_float() {
   list($usec, $sec) = explode(" ", microtime());
   return ((float)$usec + (float)$sec);
}

# mchallis added graceful error function
function gracefulerror($reason) {
    global $RSS_URL, $includeOnly, $doSummary;
    $reason = htmlspecialchars(strip_tags($reason));
    $error = ', error reading RSS Data saved';
    preg_match("|Gateway Timeout|i",$reason) and $error = ', a gateway timeout occurred';
    preg_match("|invalid zone code|i",$reason) and $error = ', invalid zone code requested';
    preg_match("|Error reading RSS data cache file|i",$reason) and $error = ', error reading cache file saved';
    !$includeOnly and $returnstring .= '<p>';
    $returnstring .= "Advisory Information Unavailable$error from the <a href=\"$RSS_URL\">NOAA advisories server.</a>\n";
    $returnstring .= "\n<!-- XML Error: $reason --></p>\n";
    !$includeOnly and $returnstring .= "</body>\n</html>\n";

    return $returnstring;
}

// ------------------------------------------------------------------

?>

